###########################################################################################
## R source code for WNV-model (Version of 29 March 2011)                                                                                 
##
## Laperriere, V., K. Brugger, and F. Rubel, 2011: Simulation of the seasonal cycles
## of bird, equine and human West Nile virus cases. Prev. Vet. Med., 98, 99-110. 
##
## (C) Institute for Veterinary Public Health, Vetmeduni Vienna, Austria
##
###########################################################################################

# required packages 
library(stats)
library(graphics)
library(grDevices)

## input data: mean monthly temperature for Minneapolis metropolitan area 
## (Minnesota. U.S.) 2002-2009

T_month=c(-2.2,-4.0,-1.7,-3.8,7.3,12.5,21.4,24.4,21.3,18.0,5.8,0.3,-3.2,
-9.0,-8.9,-0.4,8.8,14.3,19.6,22.6,23.2,16.8,10.6,0.8,-3.6,
-11.0,-5.4,2.2,9.4,13.5,18.4,21.7,18.6,19.2,9.9,3.4,-5.0, 
-9.0,-3.1,-0.3,10.6,12.7,22.0,24.0,21.4,18.5,10.7,2.7,-6.7,
-1.7,-6.8,0.9,11.5,15.9,21.0,25.4,21.8,14.8,7.5,2.4,-1.7,
-6.7,-10.7,2.9,7.8,17.4,21.9,23.5,21.3,17.4,11.5,0.7,-8.6, 
-10.6,-9.3,-2.4,6.1,13.3,19.8,23.3,21.7,16.7,9.6,1.2,-10.2,
-13.1,-6.1,-0.2,8.2,15.4,19.2,20.2,20.2,18.2,6.1,5.4,-7.9,-10.7)

midmonth=c(0,31,59,90,120,151,181,212,243,273,304,334,365,396,424,455,
485,516,546,577,608,638,669,699,730,761,790,821,851,882,
912,943,974,1004,1035,1065,1096,1127,1155,1186,1216,
1247,1277,1308,1339,1369,1400,1430,1461,1492,1520,1551,
1581,1612,1642,1673,1704,1734,1765,1795,1826,1857,1885,1916,
1946,1977,2007,2038,2069,2099,2130,2160,2191,2222,2251,
2282,2312,2343,2373,2404,2435,2465,2496,2526,2557,2588,2616,
2647,2677,2708,2738,2769,2800,2830,2861,2891,2922,2953)-15

smoothed_daily_T <- spline(midmonth, T_month, n=2922, xmin=1, xmax=2922)
T = as.vector(smoothed_daily_T$y)
n=length(T)-1
t<-seq(0,n)

# mosquito parameters

KM=3300000
k=rep(0, n); k=0.344/(1+1.231*exp(-0.184*(T-20)))
bL=rep(0, n); bL=2.325*k; bM=bL/10
mL=rep(0, n); mL=0.0025*T^2-0.094*T+1.0257; mM=mL/10
phi=45*pi/180
day <- c(rep(seq(1,365),2),seq(1,366),rep(seq(1,365),3),seq(1,366),rep(seq(1,365))) #bissextile years: 2004, 2008
epsilon=rep(0, n); epsilon=0.409*sin(2*pi*(day-80)/365)
D=rep(0, n); D=7.639*asin(tan(epsilon)*tan(phi)+0.0146/(cos(epsilon)*cos(phi)))+12
dM=rep(0, n); dM=1-1/(1+1775.7*exp(1.559*(D-18.177)))

gammaM=rep(0, n+1)
for (i in 1:n){if (T[i]>15) gammaM[i]=0.0093*T[i]-0.1352}

# bird parameters

KB=110000
phiB=30
bB=rep(0, n+1)
bB=dgamma(day, scale=1.4, shape=86.4)
mB=0.00034
alphaB=0.4 
gammaB=1.0 
nuB=0.7

# equid parameters

phiE=300
gammaE=0.05 
alphaE=0.2 
nuE=0.04
bE=0.00016
mE=0.00011

# human parameters

phiH=0.03
gammaH=0.25 
alphaH=0.5
nuH=0.004
bH=0.000055
mH=0.000034
rH=bH-mH

# transmission parameters

pM=1.0
pB=0.125

betaM=rep(0, n); betaM=k*pM
betaB=rep(0, n); betaB=k*pB

# initialisation

LM=rep(0, n+1); SM=rep(500000, n+1); EM=rep(0, n+1); IM=rep(100, n+1)
NMmin=SM[1]
NM=SM+EM+IM	

SB=rep(110000, n+1); EB=rep(0, n+1); IB=rep(0, n+1); RB=rep(0, n+1); DB=rep(0, n+1)
NB=SB+EB+IB+RB

SE=rep(10700, n+1); EE=rep(0, n+1); IE=rep(0, n+1); RE=rep(0, n+1); DE=rep(0, n+1)
NE=SE+EE+IE+RE

SH=rep(2700000, n+1); EH=rep(0, n+1); IH=rep(0, n+1); RH=rep(0, n+1); DH=rep(0, n+1)
NH=SH+EH+IH+RH

# epidemic model

for (i in 1:n){
  lambdaBM=dM[i]*betaB[i]*IB[i]/KB
  lambdaMB=dM[i]*betaM[i]*phiB*IM[i]/KM
  lambdaME=dM[i]*betaM[i]*phiE*IM[i]/KM
  lambdaMH=dM[i]*betaM[i]*phiH*IM[i]/KM


  LM[i+1]=LM[i] + (bL[i]*dM[i]*NM[i]-mL[i]*LM[i])*(1-LM[i]/KM)-bM[i]*LM[i]
  SM[i+1]=SM[i] - lambdaBM*SM[i]+bM[i]*LM[i]-mM[i]*SM[i]
  EM[i+1]=EM[i] + lambdaBM*SM[i]-gammaM[i]*EM[i]-mM[i]*EM[i]
  IM[i+1]=IM[i] + gammaM[i]*EM[i]-mM[i]*IM[i]
  NM[i+1]=SM[i+1]+EM[i+1]+IM[i+1]
  if (NM[i+1]<NMmin) {SM[i+1]=SM[i]; EM[i+1]=EM[i]; IM[i+1]=IM[i]}
  if (LM[i+1]<0) {LM[i+1]=0}

  SB[i+1]=SB[i] + (bB[i]-(bB[i]-mB)*NB[i]/KB)*NB[i]-lambdaMB*SB[i]-mB*SB[i]
  EB[i+1]=EB[i] + lambdaMB*SB[i]-gammaB*EB[i]-mB*EB[i]
  IB[i+1]=IB[i] + gammaB*EB[i]-alphaB*IB[i]-mB*IB[i]  
  RB[i+1]=RB[i] + (1-nuB)*alphaB*IB[i]-mB*RB[i]
  DB[i+1]=DB[i] + nuB*alphaB*IB[i]
  NB[i+1]=SB[i+1]+EB[i+1]+IB[i+1]+RB[i+1]  
  
  SE[i+1]=SE[i] + bE*NE[i]-lambdaME*SE[i]-mE*SE[i]
  EE[i+1]=EE[i] + lambdaME*SE[i]-gammaE*EE[i]-mE*EE[i]
  IE[i+1]=IE[i] + gammaE*EE[i]-alphaE*IE[i]-mE*IE[i]
  RE[i+1]=RE[i] + (1-nuE)*alphaE*IE[i]-mE*RE[i]
  DE[i+1]=DE[i] + nuE*alphaE*IE[i]
  NE[i+1]=SE[i+1]+EE[i+1]+IE[i+1]+RE[i+1]
  
  SH[i+1]=SH[i] + rH*NH[i]-lambdaMH*SH[i]
  EH[i+1]=EH[i] + lambdaMH*SH[i]-gammaH*EH[i]
  IH[i+1]=IH[i] + gammaH*EH[i]-alphaH*IH[i]
  RH[i+1]=RH[i] + (1-nuH)*alphaH*IH[i]
  DH[i+1]=DH[i] + nuH*alphaH*IH[i]
  NH[i+1]=SH[i+1]+EH[i+1]+IH[i+1]+RH[i+1]
}

# output graphics

if(.Platform$OS.type=="windows") {quartz<-function(...) windows(...)}
quartz(height=7, width=7); par(mfrow=c(4, 2), mar=c(1.5, 5, 0.5, 0.5), oma=c(1,0,0,0))

endyear=c(0,364,729,1095,1460,1825,2190,2556,2921)
midyear=c(182,546.5,912,1277.5,1642.5,2007.5,2373,2738.5)
yearname=c("2002", "2003", "2004", "2005", "2006", "2007", "2008", "2009")

plot (t, LM/10^6, type="l", ylim=c(0, max(SM/10^6)*1.15), xlab="", ylab=expression("mosquitoes 10"^"6"), lty=2, lwd=1.5, xaxs="i", yaxs="i", las=1, xaxt="n", cex.axis=1.2, cex.lab=1.5, mgp=c(3.5,1,0)) 
axis(1, at=endyear, labels=FALSE)
lines(t, SM/10^6, lty=1, lwd=1.5)
abline(h=KM/10^6, lty=4)
legend(2900,6.6,legend=c(expression("L"[M]),expression("S"[M]),expression("K"[M])), lwd=1.5, lty=c(2,1,4), cex=1.2, horiz=TRUE, bty="n",xjust=1)

plot (t, IM/10^4, type="l", ylim=c(0,max(IM/10^4)*1.1), xlab="", ylab=expression("mosquitoes 10"^"4"), lty=1, lwd=1.5, xaxs="i", yaxs="i", las=1, xaxt="n", cex.axis=1.2, cex.lab=1.5, mgp=c(3.5,1,0)) 
axis(1, at=endyear, labels=FALSE)
legend(2900,3.2,legend=expression("I"[M]),lty=1, lwd=1.5, bty="n", cex=1.2, xjust=1)

plot (t, SB/10^4, type="l", ylim=c(0, 12), xlab="", ylab=expression("birds 10"^"4"), lty=2, lwd=1.5, xaxs="i", yaxs="i", las=1, xaxt="n", cex.axis=1.2, cex.lab=1.5, mgp=c(3.5,1,0)) 
axis(1, at=endyear, labels=FALSE)
lines(t, RB/10^4, lty=3, lwd=1.5)
lines(t, NB/10^4, lty=1, lwd=1.5)
abline(h=KB/10^4, lty=4)
legend(2900,11,legend=c(expression("S"[B]),expression("R"[B]),expression("N"[B])), lwd=1.5, lty=c(2,3,1), cex=1.2, horiz=TRUE, bty="n",xjust=1)

plot (t, IB/10^3, type="l", ylim=c(0,max(IB/10^3)*1.1), xlab="", ylab=expression("birds 10"^"3"), lty=1, lwd=1.5, xaxs="i", yaxs="i", las=1, xaxt="n", cex.axis=1.2, cex.lab=1.5, mgp=c(3.5,1,0)) 
axis(1, at=endyear, labels=FALSE) 
legend(2900,3.2,legend=expression("I"[B]),lty=1, lwd=1.5, bty="n", cex=1.2,xjust=1)

plot (t, SE/10^3, type="l", ylim=c(0, 13), xlab="", ylab=expression("equids 10"^"3"), lty=2, lwd=1.5, xaxs="i", yaxs="i", las=1, xaxt="n", cex.axis=1.2, cex.lab=1.5, mgp=c(3.5,1,0)) 
axis(1, at=endyear, labels=FALSE)
lines(t, RE/10^3, lty=3, lwd=1.5)
lines(t, NE/10^3, lty=1, lwd=1.5)
legend(2900,10.5,legend=c(expression("S"[E]),expression("R"[E]),expression("N"[E])), lwd=1.5, lty=c(2,3,1), cex=1.2, horiz=TRUE, bty="n",xjust=1)

plot (t, IE/10^3, type="l", ylim=c(0,max(IE/10^3)*1.1), xlab="", ylab=expression("equids 10"^"3"), lty=1, lwd=1.5, xaxs="i", yaxs="i", las=1, xaxt="n", cex.axis=1.2, cex.lab=1.5, mgp=c(3.5,1,0)) 
axis(1, at=endyear, labels=FALSE) 
legend(2900,0.85,legend=expression("I"[E]),lty=1, lwd=1.5, bty="n", cex=1.2,xjust=1)

plot (t, log(SH), type="l", ylim=c(0, max(log(NH))*1.2), xlab="", ylab="humans", lty=2, lwd=1.5, xaxs="i", yaxs="i", las=1, xaxt="n", yaxt="n", cex.axis=1.2, cex.lab=1.5, mgp=c(3.5,1,0)) 
axis(1, at=midyear, labels=yearname, tick=0, cex.axis=1.2)
axis(1, at=endyear, labels=FALSE) 
axis(2, at= c(0,2.3,4.6,6.9,9.2,11.5,13.8,16.1), labels=c(0,expression("10"^"1"),expression("10"^"2"),expression("10"^"3"),expression("10"^"4"),expression("10"^"5"),expression("10"^"6"),expression("10"^"7")), las=2, cex.axis=1.2)
lines(t, log(RH), lty=3, lwd=1.5)
lines(t, log(NH), lty=1, lwd=1.5)
legend(2900,18.0,legend=c(expression("S"[H]),expression("R"[H]),expression("N"[H])), lwd=1.5, lty=c(2,3,1), cex=1.2, horiz=TRUE, bty="n",xjust=1)

plot (t, IH, type="l", ylim=c(0,max(IH)*1.2), xlab="", ylab="humans", lty=1, lwd=1.5, xaxs="i", yaxs="i", las=1, xaxt="n", cex.axis=1.3, cex.lab=1.5, mgp=c(3.5,1,0)) 
axis(1, at=midyear, labels=yearname, tick=0, cex.axis=1.2)
axis(1, at=endyear, labels=FALSE) 
legend(2900,120,legend=expression("I"[H]),lty=1, lwd=1.5, bty="n", cex=1.3,xjust=1)


